@@ -4,17 +4,15 @@ import numpy as np
 
 from scipy.sparse.linalg import eigsh
 
-from .decorators import linear, real, idempotent, symmetric, inplace
+from .decorators import linear, real, symmetric, inplace
 from .core import (Operator, CompositionOperator, DiagonalOperator,
-                   IdentityOperator, ZeroOperator, DirectOperatorFactory,
-                   ReverseOperatorFactory, asoperator)
+                   DirectOperatorFactory, ReverseOperatorFactory, asoperator)
 from .utils import isscalar, strshape
 
 __all__ = [
     'BandOperator',
     'DenseOperator',
     'EigendecompositionOperator',
-    'MaskOperator',
     'PackOperator',
     'SymmetricBandOperator',
     'TridiagonalOperator',
@@ -72,41 +70,6 @@ class DenseOperator(Operator):
         return self.data
 
 
-@real
-@idempotent
-@inplace
-class MaskOperator(DiagonalOperator):
-    """
-    A subclass of DiagonalOperator with booleans on the diagonal.
-
-    Exemple
-    -------
-    >>> M = MaskOperator([True, False])
-    >>> M.todense()
-
-    array([[False, False],
-           [False,  True]], dtype=bool)
-
-    Notes
-    -----
-    We follow the convention of MaskedArray, where True means masked.
-    """
-    def __new__(cls, data, broadcast='disabled', shapein=None, dtype=None,
-                **keywords):
-        data = np.array(data, dtype, copy=False)
-        if shapein is None and broadcast == 'disabled' and data.ndim > 0:
-            shapein = data.shape
-        if np.all(data == 1):
-            return ZeroOperator(shapein=shapein, dtype=dtype, **keywords)
-        elif np.all(data == 0):
-            return IdentityOperator(shapein=shapein, dtype=dtype, **keywords)
-        return DiagonalOperator.__new__(cls)
-
-    def __init__(self, mask, dtype=None, **keywords):
-        DiagonalOperator.__init__(self, mask, dtype=np.bool8, **keywords)
-        self.data = ~self.data
-
-
 @linear
 @real
 @inplace
