@@ -1,9 +1,12 @@
 import numpy as np
 
-from mpi4py import MPI
-from pyoperators.core import Operator
-from pyoperators.decorators import (real, linear, square, inplace)
-from pyoperators.utils import tointtuple
+try:
+    from mpi4py import MPI
+except ImportError:
+    from .utils.mpi import MPI
+from .core import Operator
+from .decorators import (real, linear, square, inplace)
+from .utils.mpi import as_mpi, distribute_shape, distribute_slice
 
 __all__ = ['DistributionGlobalOperator',
            'DistributionIdentityOperator']
@@ -35,7 +38,7 @@ class DistributionGlobalOperator(Operator):
     Given the file 'example_dgo.py':
 
     import numpy as np
-    from tamasis import *
+    from pyoperators import DistributionGlobalOperator
     from mpi4py import MPI
     x_global = np.array([1,2,3])
     d = DistributionGlobalOperator(x_global.shape)
@@ -69,9 +72,8 @@ class DistributionGlobalOperator(Operator):
             counts.append(n)
             offsets.append(offsets[-1] + n)
         offsets.pop()
-        Operator.__init__(self, shapein=shapein, shapeout=shapeout, **keywords)
-        self.commin = MPI.COMM_SELF
-        self.commout = commout
+        Operator.__init__(self, commin=MPI.COMM_SELF, commout=commout,
+                          shapein=shapein, shapeout=shapeout, **keywords)
         self.slice = slice_
         self.counts = counts
         self.offsets = offsets
@@ -120,7 +122,7 @@ class DistributionIdentityOperator(Operator):
     Given the file 'example_dio.py':
 
     import numpy as np
-    from tamasis import *
+    from pyoperators import DistributionIdentityOperator
     from mpi4py import MPI
     x_global = np.array([1,1,1])
     d = DistributionIdentityOperator()
@@ -138,9 +140,8 @@ class DistributionIdentityOperator(Operator):
     """
 
     def __init__(self, commout=None, **keywords):
-        Operator.__init__(self, **keywords)
-        self.commin = MPI.COMM_SELF
-        self.commout = commout or MPI.COMM_WORLD
+        Operator.__init__(self, commin=MPI.COMM_SELF,
+                          commout=commout or MPI.COMM_WORLD, **keywords)
 
     def direct(self, input, output):
         if self.same_data(input, output):
@@ -150,45 +151,4 @@ class DistributionIdentityOperator(Operator):
     def transpose(self, input, output):
         if not self.same_data(input, output):
             output[...] = input
-        self.commout.Allreduce(MPI.IN_PLACE, output, op=MPI.SUM)
-
-
-def distribute_shape(shape, rank=None, size=None, comm=None):
-    """
-    Return the shape of a local array given the shape of a global array,
-    according to the rank of the MPI job, The load is distributed along
-    the first dimension.
-    """
-    if rank is None or size is None:
-        comm = comm or MPI.COMM_WORLD
-    if size is None:
-        size = comm.size
-    if rank is None:
-        rank = comm.rank
-
-    shape = tointtuple(shape)
-    if len(shape) == 0:
-        if size > 1:
-            raise ValueError('It is ambiguous to split a scalar across processe'
-                             's.')
-        return ()
-    nglobal = shape[0]
-    nlocal = nglobal // size + ((nglobal % size) > rank)
-    return (nlocal,) + tuple(shape[1:])
-
-
-def distribute_slice(nglobal, rank=None, size=None, comm=None):
-    """
-    Given a number of ordered global work items, return the slice that brackets
-    the items distributed to a local MPI job.
-    """
-    if rank is None or size is None:
-        comm = comm or MPI.COMM_WORLD
-    if size is None:
-        size = comm.size
-    if rank is None:
-        rank = comm.rank
-    nlocal = nglobal // size + ((nglobal % size) > rank)
-    start = nglobal // size * rank + min(rank, nglobal % size)
-    stop = start + nlocal
-    return slice(start, stop)
+        self.commout.Allreduce(MPI.IN_PLACE, as_mpi(output), op=MPI.SUM)
